% Copyright (C) 2001, James B. Rawlings and John G. Ekerdt
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2, or (at
% your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; see the file COPYING.  If not, write to
% the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
% MA 02111-1307, USA.

% This program solves part of ethane pyrolysis example that is in the
% text.  It generates the plot of the exact solution
%  It is titled ethane_exact.m
%
% It was last edited 2/5/97
%
% Revised 7/24/2018

%E in Joules, mass in grams, T in Kelvin, time in sec, volume in cm3

Components_1 = [' C2H6 = 1,',' CH3 = 2,',' CH4 = 3,',' C2H5 = 4'];
Components_2 = [' H = 5,',' C2H4 = 6,', ' H2 = 7,',' H2O = 8'];

Ao = [1e17, 2e11, 3e14, 3.4e12, 1.6e13]';
Ea = [356000, 44000, 165000, 28000, 0]';
nu = [-1,2,0,0,0,0,0,0
      -1,-1,1,1,0,0,0,0
       0,0,0,-1,1,1,0,0
      -1,0,0,1,-1,0,1,0
       1,0,0,-1,-1,0,0,0];

R = 8.3144;  % Gas Constant (J/gmole-K)
R1 = 82.057; % Gas Constant (cc-atm/gmole-K)
T = 925; % Temp (K)
EXP = exp(-Ea/(R*T));
k = Ao.*EXP;
kp = (k(1)/(2*k(3)) + ((k(1)/(2*k(3)))^2 + ...
     ((k(1)*k(4))/(k(3)*k(5))))^0.5);
C1o = (50/760)/(82.057*T);  %gmole/cm3
C8o = (710/760)/(82.057*T);
Qf = 35.0;	%cc/sec
N1o = C1o*Qf;	%gmole/sec
N8o = C8o*Qf;
P = 1.0;	%atm
Initial = [N1o,0,0,0,0,0,0,N8o]';


v = [0:1:100]';
p = struct(); % Create structure to pass parameters to ode15s function
p.k = k;
p.T = T;
p.P = P;
p.R1 = R1;

opts = odeset ('AbsTol', sqrt (eps), 'RelTol', sqrt (eps));
[tsolver, solution] = ode15s(@(v,x) rxrate(v,x,p),v,Initial,opts);
answer = [v solution];
methane = solution(:,3);

ethane = solution(:,1);
ethylene = solution(:,6);
hydrogen = solution(:,7);
stable=[v ethane ethylene hydrogen methane];

temp = [v, solution];

save -ascii ethane_exact.dat temp;

if (~ strcmp (getenv ('OMIT_PLOTS'), 'true')) % PLOTTING
    subplot (2, 1, 1);
    plot (temp(:,1),[temp(:,7),temp(:,2)]);
    % TITLE ethane_exact

    subplot (2, 1, 2);
    plot (temp(:,1),[temp(:,4)]);
    % TITLE ethane_exact
end % PLOTTING