% Copyright (C) 2001, James B. Rawlings and John G. Ekerdt
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2, or (at
% your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; see the file COPYING.  If not, write to
% the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
% MA 02111-1307, USA.
%
% Revised 8/16/2018

a = 5;
b = 0.05;
c0 = 5;
conv = 0.95;
d  = sqrt((1 - 2*b)*(1 - 2*b) - 4*b*(b + 1));
w1 = (1-2*b + d)/(2*b);
w2 = (1-2*b - d)/(2*b);
x1  = sqrt(w1/a);
x2  = sqrt(w2/a);


c = linspace(0.01,10,300)';
y = 1./rxrate(c, a, b);

% find the outlet of the first PFR, c1, which satisfies r(c1)=1/x2

fixrate = rxrate(x2, a, b);
x0 = c0;
[x, fval, info] = fsolve(@(c) solverate(c, a, b, fixrate), x0);
info;
c1 = x;

% size the first PFR, theta1 is size such that c(theta1) = c1

t0 = 0;
ncs = 10;
cout = linspace(c0,c1,ncs);
opts = odeset('AbsTol', sqrt(eps),'RelTol', sqrt(eps));
[dummy,tout] = ode15s(@(c, t) pfr(c, t, a, b), cout, t0, opts);
theta = struct();
theta.n1 = tout(ncs);

% size the cstr, theta2= (c1-c2)/r(c2), c2 = x2
c2 = x2;
theta.n2 = (c1 - c2)/rxrate(c2, a, b);

% size the second PFR, c3, which satisfies c3 = (1-conv)*c0
c3 = (1 - conv)*c0;
t0 = 0;
ncs = 10;
cout = linspace(c2,c3,ncs);
opts = odeset('AbsTol', sqrt(eps), 'RelTol', sqrt(eps));
[dummy,tout] = ode15s(@(c, t) pfr(c, t, a, b), cout, t0, opts);
theta.n3 = tout(ncs);

% c0; c1; c2; c3;
% theta1; theta2; theta3;
% optimal reactor is PFR --->    CSTR --->  PFR
%                    theta1      theta2     theta3

% now compute segregated reactor conversion


npts = 50;
zseg = linspace(0,1,npts);
cseg0 = [c0; 0];
[dummy,cseg] = ode15s(@(z, x) seg(z, x, a, b, theta), zseg, cseg0);
csegfin = cseg(npts,2);

% solve for cinf, which seems to be the same as what would come out of
% a cstr of theta2 with feed conc c0

x0 = c0;
[x, fval, info] = fsolve(@(c) climit(c, a, b, theta, c0), x0);
info;
cinf = x;

% now compute maximally mixed conversion

npts = 50;
z = linspace(0, 1, npts);
[dummy,cmm] = ode15s(@(z, c) mm(z, c, a, b, theta, c0), z, cinf);
cmmfin = cmm(npts);
table = [c y];
extras = [c0 1/rxrate(c0, a, b) c1 1/rxrate(c1, a, b) c2 1/rxrate(c2, a, b)...
          c3 1/rxrate(c3, a, b) c1 1/rxrate(c1, a, b) ;
          c0 0                  c1 0                  c2 0          ...
          c3 0                  c2 1/rxrate(c2, a, b)];

save leven.dat table extras;

if (~ strcmp (getenv ('OMIT_PLOTS'), 'true')) % PLOTTING
    plot (table(:,1), table(:,2), ...
           extras(:,1), extras(:,2), ...
           extras(:,3), extras(:,4), ...
           extras(:,5), extras(:,6), ...
           extras(:,7), extras(:,8), ...
           extras(:,9), extras(:,10));
    axis ([0,6,0,10])
    % TITLE
end % PLOTTING